%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Memory_update.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
% DESCRIPTION:
%   Memory update routine
%
% INPUT:
%
% OUTPUT:
%
% USAGE:
% 
% SUBROUTINE CALLS: none
%      
%
function [x_mat, f_vec, gradf_mat, ngradf_vec, stepsize_vec, mem] = ...
    Memory_update(x_mat, f_vec, gradf_mat, ngradf_vec,...
    stepsize_vec, mem, maxmem, psd,iter, accept_step)

% disp('In Memory_update')
% keyboard

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Update the memory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(psd==1)
    % extra check that sty>0 for BFGS-based methods
    s_vec = x_mat(:,maxmem+1)-x_mat(:,maxmem);
    y_vec = gradf_mat(:,maxmem+1)-gradf_mat(:,maxmem);
    sty=s_vec'*y_vec;
    sty_tol=norm(y_vec)*norm(s_vec)*1e-12;
    
    if(sty<=sty_tol)
        % lost positive defniniteness:  delete old memory elements
        if (accept_step==1)
            % still made progress, so keep new point and reset history
            mem                     = 0;
            x_mat(:,maxmem)         = x_mat(:,maxmem+1);
            gradf_mat(:,maxmem)     = gradf_mat(:,maxmem+1);
            ngradf_vec(iter+1)      = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));
        elseif(accept_step==0)
            % did not make progress, reset history starting from
            % current point
            mem                     = 0;
            ngradf_vec(iter+1)      = ngradf_vec(iter);
        end
    else
        if(accept_step==0)
            % No progress, but keep new information.
            % Recenter the steps: keep the current step the center
            %    and move the new information back in the memory
            tmp                     = f_vec(iter+1);
            f_vec(iter+1)           = f_vec(iter);
            f_vec(iter)             = tmp;
            ngradf_vec(iter+1)      = ngradf_vec(iter);
            ngradf_vec(iter)        = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));

            % update memory
            mem                     = min(mem+1,maxmem-1);
            x_mat(:,1:maxmem-1)     = x_mat(:,2:maxmem);
            x_mat(:,maxmem)         = x_mat(:,maxmem+1);
            gradf_mat(:,1:maxmem-1) = gradf_mat(:,2:maxmem);
            gradf_mat(:,maxmem)     = gradf_mat(:,maxmem+1);
            ngradf_vec(iter+1)      = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));

        elseif(accept_step==-1)
            % No progress, don't take the step and shrink memory.
            % update memory
            mem                     = max(mem-1,0);
            ngradf_vec(iter+1)      = ngradf_vec(iter);
            % xold_vec, fold, and gradf_vec do not change
        elseif(accept_step==-2)
            % did not make progress, reset history starting from
            % current point
            mem                     = 0;
            ngradf_vec(iter+1)      = ngradf_vec(iter);
        elseif(accept_step==1)
            % step accepted --> model working
            % take the proposed step and update memory 
            mem                     = min(mem+1,maxmem-1);
            x_mat(:,1:maxmem-1)     = x_mat(:,2:maxmem);
            x_mat(:,maxmem)         = x_mat(:,maxmem+1);
            gradf_mat(:,1:maxmem-1) = gradf_mat(:,2:maxmem);
            gradf_mat(:,maxmem)     = gradf_mat(:,maxmem+1);
            ngradf_vec(iter+1)      = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));
        end
    end
elseif(accept_step==0)
    % step not acctepted --> past information
    % is no good, but don't throw away new information.
    % Recenter the steps: keep the current step the center
    %    and move the new information back in the memory
    tmp                     = f_vec(iter+1);
    f_vec(iter+1)           = f_vec(iter);
    f_vec(iter)             = tmp;
    ngradf_vec(iter+1)      = ngradf_vec(iter);
    ngradf_vec(iter)        = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));

    % update memory
    mem                     = min(mem+1,maxmem-1);
    x_mat(:,1:maxmem-1)     = x_mat(:,2:maxmem);
    x_mat(:,maxmem)         = x_mat(:,maxmem+1);
    gradf_mat(:,1:maxmem-1) = gradf_mat(:,2:maxmem);
    gradf_mat(:,maxmem)     = gradf_mat(:,maxmem+1);
    ngradf_vec(iter+1)      = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));
    % % mem does not change, effectively deletes old data
    % x_mat(:,1:maxmem-2)     = x_mat(:,2:maxmem-1);
    % x_mat(:,maxmem-1)       = x_mat(:,maxmem+1);
    % gradf_mat(:,1:maxmem-2) = gradf_mat(:,2:maxmem-1);
    % gradf_mat(:,maxmem-1)   = gradf_mat(:,maxmem+1);
    % xold_vec, fold, and gradf_vec do not change
elseif(accept_step==-1)
    % No progress, don't take the step and shrink memory.
    % update memory
    mem                     = max(mem-1,0);
    ngradf_vec(iter+1)      = ngradf_vec(iter);
    % xold_vec, fold, and gradf_vec do not change
elseif(accept_step==-2)
    % did not make progress, reset history starting from
    % current point
    mem                     = 0;
    ngradf_vec(iter+1)      = ngradf_vec(iter);
elseif(accept_step==1)
    % model working
    % take the proposed step and update memory
    mem                     = min(mem+1,maxmem-1);
    x_mat(:,1:maxmem-1)     = x_mat(:,2:maxmem);
    x_mat(:,maxmem)         = x_mat(:,maxmem+1);
    gradf_mat(:,1:maxmem-1) = gradf_mat(:,2:maxmem);
    gradf_mat(:,maxmem)     = gradf_mat(:,maxmem+1);
    ngradf_vec(iter+1)      = sqrt(gradf_mat(:,maxmem+1)'*gradf_mat(:,maxmem+1));
end
stepsize_vec(iter+1)        = stepsize_vec(iter);
%-----------------------------------------
%  End update the memory
%-----------------------------------------

clear maxmem eta3 psd iter tmp s_vec y_vec sty sty_tol

return