%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Dogleg_QN.m
%                   Russell Luke
%               Department of MathSci
%              University of Delaware  
%                  April 12, 2008 
%
%
%
%
% DESCRIPTION:  TO DO
% 
%  Stable Quasi-Newton Dogleg
%
% INPUT:
%       TO DO
%
% OUTPUT:
%        TO DO
%
% USAGE: 
%
%       d=Dogleg_QN(Hessian_product,invHessian_product,dim,mem,grad_vec,ngrad,...
%                   S_mat, Y_mat, tr, del,alpha)
%
% SUBROUTINE CALLS:
%      none
%      
% Dogleg subroutine for line search methods

function d = Dogleg_QN(Hessian_product, invHessian_product,dim,mem,grad_vec, ngrad, S_mat, Y_mat, tr, del,alpha,verbose)

% disp(Hessian_product)
% disp(invHessian_product)
% disp(dim)
% disp(mem)
% disp(grad_vec)
% disp(ngrad)
% disp(S_mat)
% disp(Y_mat)
% disp(tr)
% disp(del)
% disp(alpha)
dn = feval(invHessian_product,dim,mem,-grad_vec, S_mat, Y_mat,del, ...
           alpha);
disp('--- in Dogleg_QN.m ---')
disp('dn =')
disp(dn)
disp('--- out Dogleg_QN.m ---')
if norm(dn) <= tr % to turn on  set <= 1e+15
    ddl = dn;
else
    % disp('in dogleg')
    % initialization
    u = -grad_vec/ngrad;
    v = feval(Hessian_product,dim,mem, u, S_mat, Y_mat,del, alpha);
    w = u'*v;
    b = -tr*ngrad + .5*(tr^2)*w;  % predicted change at constrained solution
    t_star = ngrad/w;                % t_star solves min{-t||grad_f||
    if (t_star < 0) || (t_star > tr) %  + 1/2*t^2* grad_f'*H*grad_f/||grad_f||^2}
        if b > 0                       % => t_star = ||grad_f||^3/grad_f'*H*grad_f
            t = 0;                       % from first order necessary conditions for
        else                           % optimality.
            t = tr;
        end
    else
        c = -.5*(ngrad^2)/w; % predicted change at unconstrained solution
        if c < 0 || b < 0   %  Only want the t that solves
            if c < b       %  min{-t||grad_f||+ 1/2*t^2* grad_f'*H*grad_f/||grad_f||^2}
                t = t_star;  % on [0,tr].
            else
                t = tr;
            end
        else
            t = 0;
        end
    end
    dsd = t*u;

    tmp = norm(dsd-dn)^2;
    disc = ((dsd-dn)'*dsd)^2 + tmp*(tr^2 - dsd'*dsd);
    % disp('disc')
    % disp(disc)
    lamb = max((sqrt(disc)-(dn-dsd)'*dsd)/tmp,0);
    ddl = lamb*dn + (1-lamb)*dsd;
end
d = ddl;
