%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    FFT.m
%                   Russell Luke
%             Department of MathSci
%              University of Delaware  
%                    Oct 3, 2007 
% Fourier transform processor.   FFT's data in the physical domain and
% adjusts for the fft algorithm peculiarities (which have more to do with 
% one's own misunderstandings, but since these are harder to correct...).
% First we fft the data, then we take the negative sign of every other
% mode in each direction (x and y) to shift the phase, then we
% divide by (sqrt(res))^2 to get the correct intensity scaling,
% finally we fftshift everything.

function F=FFT(f)

[ydim,xdim]=size(f);
res=max(xdim,ydim);

if (xdim == ydim)
  F=feval('fft2',f);
  F=F/(res);
  F(2:2:res,:)=-F(2:2:res,:);
  F(:,2:2:res)=-F(:,2:2:res);
else
  F=feval('fft',f);
  F(2:2:res)=-F(2:2:res);
  F=F/sqrt(res);
end

F=feval('fftshift',F);
