function [Data_params]=data_processor(newres,noise)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 data_processor.m
%                   Russell Luke
%               Department of MathSci
%               University of Delaware  
%                    Oct 3, 2007 
%
%
%
%
% DESCRIPTION:
% 
%     Function for processing wavefront recovery data from the
%     Jame Webb Space Telescope (JWST) prototype.  
%
% INPUT:
%
%     newres:  integer-valued number for resolution of data.
%              Recommended to make this either 32, 64, 128, 256 or
%              512 (max)
%     noise:  0 or 1 switch for generating noisy data.
%
% OUTPUT:
%        Data_params: at structured array containing data
%                     parameters necessary for the evaluation of
%                     the objective function J_gradJ.m
%
% USAGE: 
%
%       [Data_params]=data_processor(newres,noise)
% 
% DATA Files read: 
%       ./data/pupil.pmod
%       ./data/phase_m37.pmod
%       ./data/phase_p37.pmod
%
% SUBROUTINE CALLS:
%       Resize.m
%       FFT.m
%       IFFT.m
%       MgenN.m

im=sqrt(-1);

[fid,message]=fopen('./data/pupil.pmod','r','l');
if (fid<0)
  disp(message)
else
  Xi_A=fread(fid,[512,512],'float');
  fclose(fid);
end

diversity=3;

[fid,message]=fopen('./data/phase_p37.pmod','r','l');
if (fid<0)
  disp(message)
else
  temp1=fread(fid,[512,512],'float');
  fclose(fid);
end

[fid,message]=fopen('./data/phase_m37.pmod','r','l');
if (fid<0)
  disp(message)
else
  temp2=fread(fid,[512,512],'float');
  fclose(fid);
end

defocus=(temp1-temp2)/2;
theta=(temp1+temp2)/2;
if newres~=512
  defocus=feval('Resize',defocus,newres,newres);
  theta=feval('Resize',theta,newres,newres);
  Xi_A=feval('Resize',Xi_A,newres,newres);
end
aberration(:,:,1)=zeros(newres,newres);  % Note this order!!!!!
aberration(:,:,2)=defocus;  % Note this order!!!!!
aberration(:,:,3)=-defocus;  % Note this order!!!!!


for j=1:diversity
  Pj=Xi_A.*exp(im*(aberration(:,:,j)+theta));
  %   Y(:,:,j)=Star(Pj,conj(Pj));
  Y(:,:,j)=feval('IFFT',(feval('IFFT',Pj).*feval('FFT',feval('conj',Pj))));
end

clear Pj temp1 temp2 defocus theta

epsilon=0;

for i=1:diversity
  temp=Y(:,:,i);
  temp=feval('abs',feval('IFFT',temp));
  I=find(temp<=epsilon);
  temp(I)=epsilon;
  k(:,:,i)=temp;
end

clear I Y epsilon i

if noise==1
  k1=k(:,:,1)+feval('MgenN',newres,newres,.0625,.125,3);
  temp=find(k1<=0);
  k1(temp)=0;
  k2=k(:,:,2)+feval('MgenN',newres,newres,.0625,.125,4);
  temp=find(k2<=0);
  k2(temp)=0;
  k3=k(:,:,3)+feval('MgenN',newres,newres,.0625,.125,5);
  temp=find(k3<=0);
  k3(temp)=0;
  k(:,:,1)=k1;
  k(:,:,2)=k2;
  k(:,:,3)=k3;
else
end

k=k.^(.5);

% Normalize everything
% Xi_A=Xi_A/feval('norm',Xi_A,'fro');
% k(:,:,1)=k(:,:,1)/feval('norm',k(:,:,1),'fro');
% k(:,:,2)=k(:,:,2)/feval('norm',k(:,:,2),'fro');
% k(:,:,3)=k(:,:,3)/feval('norm',k(:,:,3),'fro');

clear k1 k2 k3 temp

Data_params.k = k;
Data_params.aberration = aberration;
Data_params.Xi_A  =Xi_A;
Data_params.diversity = diversity;



