from matlab import double
import matlab.engine

from numpy import array, zeros, load, amax, abs
from numpy.linalg import norm

import os

from scipy.io import loadmat

from samsara import Samsara, BFGS1_product, Explicit_TR, cdSY_mat

from problems.Burke.burke import Burke_func

eng = matlab.engine.start_matlab()
eng.addpath('../../../matlab')

try:
    os.remove('state.dat')
    os.remove('stephistory.dat')
except Exception:
    pass

EPS = 1e-8


def test_driver():
    Objective = Burke_func
    x0_vec = zeros(7)

    # OPTIONS
    ngrad_TOL = 1e-12
    step_TOL = 1e-14
    Maxit = 300

    s = Samsara()

    params = {}
    params['verbose'] = True
    params['alpha_0'] = 5e-8
    params['gamma'] = .5
    params['c'] = .01
    params['beta'] = .9999999
    params['eta1'] = .995
    params['eta2'] = .8
    params['eta3'] = .25
    # params['eta4'] = .025
    params['maxmem'] = 9
    # params['lookback'] = 0
    params['tr'] = 1e+15

    tols = {}
    tols['ngrad_TOL'] = 1e-12
    tols['step_TOL'] = 1e-16
    tols['Maxit'] = 500
    tols['orthog_TOL'] = 1e-6

    opts = {}
    opts['QN_method'] = BFGS1_product
    opts['Step_finder'] = Explicit_TR
    opts['History'] = cdSY_mat
    opts['update_conditions'] = 'Trust Region'
    opts['initial_step'] = 1e+5 * tols['step_TOL']

    s.load_options(params, tols, opts)

    it = 0
    xold_vec = x0_vec
    fold, gradfold_vec = Objective(xold_vec)

    xnew_vec = array([])
    stepsize = 999.
    fnew = None
    gradfnew_vec = array([])
    ngradfnew = 999.

    xnew_m = eng.transpose(double(xnew_vec.tolist()))
    xold_m = eng.transpose(double(xold_vec.tolist()))
    fold_m = fold
    fnew_m = double([])
    gradfold_m = eng.transpose(double(gradfold_vec.tolist()))
    gradfnew_m = eng.transpose(double(gradfnew_vec.tolist()))

    while it < Maxit and ngradfnew > ngrad_TOL and stepsize > step_TOL:
        # xnew_m = eng.transpose(double(xnew_vec.tolist()))
        # xold_m = eng.transpose(double(xold_vec.tolist()))
        # fold_m = fold
        # if fnew is None:
        #     fnew_m = double([])
        # else:
        #     fnew_m = fnew
        # gradfold_m = eng.transpose(double(gradfold_vec.tolist()))
        # gradfnew_m = eng.transpose(double(gradfnew_vec.tolist()))

        print(50*'#', 'ITERATION', it, 50*'#')
        print(30*'-', 'PYTHON', 30*'-')
        xnew_vec, xold_vec, fold, gradfold_vec, stepsize =\
            s.run(xold_vec, xnew_vec, fold, fnew,
                  gradfold_vec, gradfnew_vec)

        print()
        print(30*'-', 'MATLAB', 30*'-')
        xnew_m, xold_m, fold_m, gradfold_m, stepsize_m =\
            eng.samsara(xold_m, xnew_m, fold_m, fnew_m, gradfold_m, gradfnew_m,
                        nargout=5)

        # read in values from files and print out values/differences
        try:
            mat_vars = loadmat('mat_vars.mat')
            py_vars = load('py_vars.npz')
            print()
            print(30*'-', 'VARIABLES', 30*'-')
            for var in py_vars.files:
                py_var = py_vars[var]
                try:
                    mat_var = mat_vars[var]
                except Exception:
                    print('!!!', var, 'NOT IN MAT FILE !!!')
                    continue
                diff = array(mat_var).flatten() - array(py_var).flatten()
                if amax(abs(diff)) > EPS*amax(abs(array(mat_var).flatten())):
                    # if amax(abs(diff)) > EPS:
                    print('###', var, 'DIFFERS ###')
                    print('MATLAB:')
                    print(mat_vars[var])
                    print('PYTHON:')
                    print(py_vars[var])
                    print()
                else:
                    print('###', var, 'are the same ###')
        except Exception as e:
            print(e)

        # print('=== ' + str(it) + 'th iteration ===')
        # print('xnew_vec =', xnew_vec)
        # print('xold_vec =', xold_vec)
        # print('fold =', fold)
        # print('gradfold_vec =', gradfold_vec)
        # print('stepsize =', stepsize)

        it += 1
        fnew, gradfnew_vec = Objective(xnew_vec)
        ngradfnew = norm(gradfnew_vec)

        fnew_m = fnew
        gradfnew_m = eng.transpose(double(gradfnew_vec.tolist()))

        # print('fnew =', fnew)
        # print('gradfnew_vec =', gradfnew_vec)
        # print('ngradfnew =', ngradfnew)

    if stepsize <= step_TOL:
        print('Algorithm stagnated:  stepsize tolerance violated.')

    xopt_vec = xnew_vec
    fopt = fnew

    print('xopt_vec =', xopt_vec)
    print('fopt =', fopt)

    assert False

    # py.test should always report back
    # assert False
    diff = array(xnew_m).flatten() - xnew_vec
    assert amax(abs(diff)) < EPS*amax(abs(array(xnew_m).flatten()))
    return xopt_vec, fopt


if __name__ == '__main__':
    test_driver()
